#!BPY

"""
Name: 'ROFF animation (.rof)...'
Blender: 249
Group: 'Import'
Tooltip: 'Open a ROFF animation and apply it to the selected object'
"""

__author__ = "Willi \"Mr. Wonko\" Schinmeyer"
__url__ = ['www.mrwonko.de']
__version__ = "1.0"

__bpydoc__ = """\
This script is an importer for the ROFF animation file format.

Usage:

Run this script from "File->Import" menu and then select the desired roff file.
The currently selected object will then get the animations from the file applied.
"""

import Blender
import struct

SCALE = 1

def ImportStart(glmfilename):
	objlist = Blender.Object.GetSelected()
	if len(objlist) == 0:
		Blender.Draw.PupBlock("Please select something first.", ["Do it!"])
		return
	obj = objlist[0]
	ipo = obj.getIpo()
	if ipo is None:
		ipo = Blender.Ipo.New("Object", "roff_ipo")
		obj.setIpo(ipo)
	
	locx = ipo[Blender.Ipo.OB_LOCX]
	if locx is None:
		locx = ipo.addCurve("LocX")
	
	locy = ipo[Blender.Ipo.OB_LOCY]
	if locy is None:
		locy = ipo.addCurve("LocY")
	
	locz = ipo[Blender.Ipo.OB_LOCZ]
	if locz is None:
		locz = ipo.addCurve("LocZ")
	
	rotx = ipo[Blender.Ipo.OB_ROTX]
	if rotx is None:
		rotx = ipo.addCurve("RotX")
	
	roty = ipo[Blender.Ipo.OB_ROTY]
	if roty is None:
		roty = ipo.addCurve("RotY")
	
	rotz = ipo[Blender.Ipo.OB_ROTZ]
	if rotz is None:
		rotz = ipo.addCurve("RotZ")
	
	try:
		file = open(glmfilename, "rb")
		# I don't know what the last 8 bytes are, they (nearly) always contain (int) 50 and (int) 0 (once the second contained 7, once 8...)
		ident, version, frames = struct.unpack("4s2i8x", file.read(20))
		if ident != b"ROFF":
			Blender.Draw.PupBlock("That's no roff file!", ["Really!"])
			return
		if version != 2:
			Blender.Draw.PupBlock("That's not version 2, cannot load it.", ["Sorry."])
			return
		
		locx[1] = 0
		locy[1] = 0
		locz[1] = 0
		
		rotx[1] = 0
		roty[1] = 0
		rotz[1] = 0
		for frame in range(frames):
			# I don't know what the last 8 bytes are, they always contain 0xFFFFFFFF00000000 - I trash them.
			dx, dy, dz, droty, drotz, drotx = struct.unpack("6f8x", file.read(32))
			locx[frame+2] = dx*SCALE + locx[frame+1]
			locy[frame+2] = dy*SCALE + locy[frame+1]
			locz[frame+2] = dz*SCALE + locz[frame+1]
			
			rotx[frame+2] = drotx/10 + rotx[frame+1]
			roty[frame+2] = droty/10 + roty[frame+1]
			rotz[frame+2] = drotz/10 + rotz[frame+1]
	except IOError:
		Blender.Draw.PupBlock("Couldn't open file!", ["Select a real one!"]);
		return

#open the file selector
Blender.Window.FileSelector(ImportStart, 'Import a ROFF', '.rof')